from object_packaging.field import Field
from object_packaging.my_database import create_pool


# 定义一个元类，控制model对象的创建
class ModelMetaClass(type):
    def __new__(cls, table_name, bases, attrs):
        if table_name == "Model":
            return super(ModelMetaClass, cls).__new__(cls, table_name, bases, attrs)
        mappings = dict()
        for k, v in attrs.items():
            # 保存类属性和列的映射关系到mappings字典中
            if isinstance(v, Field):
                mappings[k] = v  # 这样，这个mappings就存放了  属性名称和字段名称及列的名字
        for k in mappings.keys():
            attrs.pop(k)   # 只有在实例中才可以进行访问。也就是说使用类名.属性就不能访问了
        # 把表名转换为小写，表名就是类名
        # 这个处理略显粗糙，应该继续做一些健壮性的判断，比如ModelMetaClass 变成表名应该是model_meta_class
        attrs['__table__'] = table_name.lower()
        # 保存属性和列的映射关系，创建类时添加一个__mappings__类的属性
        attrs['__mappings__'] = mappings

        return super(ModelMetaClass, cls).__new__(cls, table_name, bases, attrs)



# 编写一个model子类，这个类用于被具体的model对象继承。 它实现了具体的增删改查方法
# 这样以后的每一个model就都有了这些方法
# 这里边我又继承了dict，那么也就是说这个Model也可以使用dict的方法

class Model(dict, metaclass=ModelMetaClass):
    def __init__(self, **kwargs):
        super(Model, self).__init__(**kwargs)

    def __getattr__(self, item):
        try:
            return self[item]
        except KeyError:
            raise AttributeError("这个Model没有这个属性，这个属性是%s" % item)

    def __setattr__(self, key, value):
        self[key] = value



    def insert(self):
        fields = []
        params = []
        args = []
        for k, v in self.__mappings__.items():
            fields.append(v.column_name)
            params.append("?")
            args.append(getattr(self, k))
        sql = 'insert into %s (%s) values (%s)' % (self.__table__, ','.join(fields), ','.join(params))

        res = self.__execute(sql, args)
        print(res)

    def __execute(self, sql, args):
        try:
            connection = create_pool()
            cursor = connection.cursor()
            cursor.execute(sql.replace('?', '%s'), args)
            connection.commit()
            affected = cursor.rowcount
        finally:
            cursor.close()
            connection.close()

        return affected

# 下边是作业

    # get  select
    def select(self):
        fields = []
        params = []
        args = []
        for k, v in self.__mappings__.items():
            fields.append(v.column_name)
            params.append("?")
            args.append(getattr(self, k))
        sql = 'select * from  %s where (%s) = (%s)' % (self.__table__, ','.join(fields), ','.join(params))

        res = self.__execute(sql, args)
        print(res)


    # update
    def update(self):
        fields = []
        params = []
        args = []
        for k, v in self.__mappings__.items():
            fields.append(v.column_name)
            params.append("?")
            args.append(getattr(self, k))
        sql = 'update  %s set (%s) = (%s) where (%s) = (%s)' % (self.__table__, ','.join(fields), ','.join(params),','.join(fields), ','.join(params))

        res = self.__execute(sql, args)
        print(res)



    # delete
    def delete(self):
        fields = []
        params = []
        args = []
        for k, v in self.__mappings__.items():
            fields.append(v.column_name)
            params.append("?")
            args.append(getattr(self, k))
        sql = 'delete  from  %s where (%s) = (%s)' % (self.__table__, ','.join(fields), ','.join(params))

        res = self.__execute(sql, args)
        print(res)
